@@ -7,6 +7,7 @@ extern crate serde_json;
 use serde::Deserialize;
 use std::io::{BufReader, Read};
 use std::sync::Arc;
+use vrp_core::models::Problem;
 use vrp_core::solver::mutation::*;
 use vrp_core::solver::Builder;
 
@@ -48,7 +49,9 @@ pub enum RuinMethod {
     #[serde(rename(deserialize = "random-route"))]
     RandomRoute { probability: f64, min: usize, max: usize, threshold: f64 },
     #[serde(rename(deserialize = "worst-job"))]
-    WorstJob { probability: f64, min: usize, max: usize, threshold: usize, skip: usize },
+    WorstJob { probability: f64, min: usize, max: usize, threshold: f64, skip: usize },
+    #[serde(rename(deserialize = "cluster"))]
+    Cluster { probability: f64, min: usize, max: usize, threshold: f64, cmin: usize, cmax: usize },
 }
 
 /// Specifies recreate methods with their probability weight and specific parameters.
@@ -119,9 +122,10 @@ fn configure_from_population(mut builder: Builder, population_config: &Option<Po
 fn configure_from_mutation(mut builder: Builder, mutation_config: &Option<MutationConfig>) -> Builder {
     if let Some(config) = mutation_config {
         let MutationConfig::RuinRecreate { ruins, recreates } = config;
+        let problem = builder.config.problem.clone();
         builder = builder.with_mutation(Box::new(RuinAndRecreateMutation::new(
             Box::new(CompositeRecreate::new(recreates.iter().map(|r| create_recreate_method(r)).collect())),
-            Box::new(CompositeRuin::new(ruins.iter().map(|g| create_ruin_group(g)).collect())),
+            Box::new(CompositeRuin::new(ruins.iter().map(|g| create_ruin_group(&problem, g)).collect())),
         )));
     }
 
@@ -148,27 +152,31 @@ fn create_recreate_method(method: &RecreateMethod) -> (Box<dyn Recreate>, usize)
     }
 }
 
-fn create_ruin_group(group: &RuinMethodGroup) -> (Vec<(Arc<dyn Ruin>, f64)>, usize) {
-    (group.methods.iter().map(|r| create_ruin_method(r)).collect(), group.weight)
+fn create_ruin_group(problem: &Arc<Problem>, group: &RuinMethodGroup) -> (Vec<(Arc<dyn Ruin>, f64)>, usize) {
+    (group.methods.iter().map(|r| create_ruin_method(problem, r)).collect(), group.weight)
 }
 
-fn create_ruin_method(method: &RuinMethod) -> (Arc<dyn Ruin>, f64) {
+fn create_ruin_method(problem: &Arc<Problem>, method: &RuinMethod) -> (Arc<dyn Ruin>, f64) {
     match method {
         RuinMethod::AdjustedString { probability, lmax, cavg, alpha } => {
             (Arc::new(AdjustedStringRemoval::new(*lmax, *cavg, *alpha)), *probability)
         }
         RuinMethod::Neighbour { probability, min, max, threshold } => {
-            (Arc::new(NeighbourRemoval::new(*min, *max, *threshold)), *probability)
+            (Arc::new(NeighbourRemoval::new(JobRemovalLimit::new(*min, *max, *threshold))), *probability)
         }
         RuinMethod::RandomJob { probability, min, max, threshold } => {
-            (Arc::new(RandomJobRemoval::new(*min, *max, *threshold)), *probability)
+            (Arc::new(RandomJobRemoval::new(JobRemovalLimit::new(*min, *max, *threshold))), *probability)
         }
         RuinMethod::RandomRoute { probability, min, max, threshold } => {
             (Arc::new(RandomRouteRemoval::new(*min, *max, *threshold)), *probability)
         }
         RuinMethod::WorstJob { probability, min, max, threshold, skip: worst_skip } => {
-            (Arc::new(WorstJobRemoval::new(*threshold, *worst_skip, *min, *max)), *probability)
+            (Arc::new(WorstJobRemoval::new(*worst_skip, JobRemovalLimit::new(*min, *max, *threshold))), *probability)
         }
+        RuinMethod::Cluster { probability, min, max, threshold, cmin, cmax } => (
+            Arc::new(ClusterRemoval::new(problem.clone(), *cmin..*cmax, JobRemovalLimit::new(*min, *max, *threshold))),
+            *probability,
+        ),
     }
 }
 
@@ -178,13 +186,16 @@ pub fn read_config<R: Read>(reader: BufReader<R>) -> Result<Config, String> {
 }
 
 /// Creates a solver `Builder` from config file.
-pub fn create_builder_from_config_file<R: Read>(reader: BufReader<R>) -> Result<Builder, String> {
-    read_config(reader).and_then(|config| create_builder_from_config(&config))
+pub fn create_builder_from_config_file<R: Read>(
+    problem: Arc<Problem>,
+    reader: BufReader<R>,
+) -> Result<Builder, String> {
+    read_config(reader).and_then(|config| create_builder_from_config(problem, &config))
 }
 
 /// Creates a solver `Builder` from config.
-pub fn create_builder_from_config(config: &Config) -> Result<Builder, String> {
-    let mut builder = Builder::default();
+pub fn create_builder_from_config(problem: Arc<Problem>, config: &Config) -> Result<Builder, String> {
+    let mut builder = Builder::new(problem);
 
     builder = configure_from_population(builder, &config.population);
     builder = configure_from_mutation(builder, &config.mutation);
